@@ -9,9 +9,6 @@
 
 #![allow(non_upper_case_globals)]
 
-#[cfg(feature = "heapsize")]
-use heapsize::HeapSizeOf;
-
 use phf_shared;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
@@ -23,7 +20,6 @@ use std::hash::{Hash, Hasher};
 use std::marker::PhantomData;
 use std::mem;
 use std::ops;
-use std::ptr;
 use std::slice;
 use std::str;
 use std::sync::Mutex;
@@ -47,28 +43,10 @@ struct StringCache {
     buckets: [Option<Box<StringCacheEntry>>; NB_BUCKETS],
 }
 
-#[cfg(feature = "heapsize")]
-impl HeapSizeOf for StringCache {
-    fn heap_size_of_children(&self) -> usize {
-        self.buckets.iter().fold(0, |size, bucket| size + bucket.heap_size_of_children())
-    }
-}
-
 lazy_static! {
     static ref STRING_CACHE: Mutex<StringCache> = Mutex::new(StringCache::new());
 }
 
-/// A token that represents the heap used by the dynamic string cache.
-#[cfg(feature = "heapsize")]
-pub struct StringCacheHeap;
-
-#[cfg(feature = "heapsize")]
-impl HeapSizeOf for StringCacheHeap {
-    fn heap_size_of_children(&self) -> usize {
-        STRING_CACHE.lock().unwrap().heap_size_of_children()
-    }
-}
-
 struct StringCacheEntry {
     next_in_bucket: Option<Box<StringCacheEntry>>,
     hash: u64,
@@ -76,14 +54,6 @@ struct StringCacheEntry {
     string: Box<str>,
 }
 
-#[cfg(feature = "heapsize")]
-impl HeapSizeOf for StringCacheEntry {
-    fn heap_size_of_children(&self) -> usize {
-        self.next_in_bucket.heap_size_of_children() +
-        self.string.heap_size_of_children()
-    }
-}
-
 impl StringCacheEntry {
     fn new(next: Option<Box<StringCacheEntry>>, hash: u64, string: String)
            -> StringCacheEntry {
@@ -214,20 +184,18 @@ pub struct Atom<Static: StaticAtomSet> {
     pub phantom: PhantomData<Static>,
 }
 
-#[cfg(feature = "heapsize")]
-impl<Static: StaticAtomSet> HeapSizeOf for Atom<Static> {
-    #[inline(always)]
-    fn heap_size_of_children(&self) -> usize {
-        0
-    }
-}
-
 impl<Static: StaticAtomSet> ::precomputed_hash::PrecomputedHash for Atom<Static> {
     fn precomputed_hash(&self) -> u32 {
         self.get_hash()
     }
 }
 
+impl<'a, Static: StaticAtomSet> From<&'a Atom<Static>> for Atom<Static> {
+    fn from(atom: &'a Self) -> Self {
+        atom.clone()
+    }
+}
+
 fn u64_hash_as_u32(h: u64) -> u32 {
     // This may or may not be great...
     ((h >> 32) ^ h) as u32
@@ -269,7 +237,7 @@ impl<Static: StaticAtomSet> Default for Atom<Static> {
 impl<Static: StaticAtomSet> Hash for Atom<Static> {
     #[inline]
     fn hash<H>(&self, state: &mut H) where H: Hasher {
-        self.unsafe_data.hash(state)
+        state.write_u32(self.get_hash())
     }
 }
 
@@ -314,7 +282,7 @@ impl<'a, Static: StaticAtomSet> From<Cow<'a, str>> for Atom<Static> {
             let len = string_to_add.len();
             if len <= MAX_INLINE_LEN {
                 let mut buf: [u8; 7] = [0; 7];
-                copy_memory(string_to_add.as_bytes(), &mut buf);
+                buf[..len].copy_from_slice(string_to_add.as_bytes());
                 Inline(len as u8, buf)
             } else {
                 Dynamic(STRING_CACHE.lock().unwrap().add(string_to_add, hash) as *mut ())
@@ -528,18 +496,31 @@ enum UnpackedAtom {
     Static(u32),
 }
 
-struct RawSlice {
-    data: *const u8,
-    len: usize,
+#[inline(always)]
+fn inline_atom_slice(x: &u64) -> &[u8] {
+    unsafe {
+        let x: *const u64 = x;
+        let mut data = x as *const u8;
+        // All except the lowest byte, which is first in little-endian, last in big-endian.
+        if cfg!(target_endian = "little") {
+            data = data.offset(1);
+        }
+        let len = 7;
+        slice::from_raw_parts(data, len)
+    }
 }
 
-#[cfg(target_endian = "little")]  // Not implemented yet for big-endian
 #[inline(always)]
-unsafe fn inline_atom_slice(x: &u64) -> RawSlice {
-    let x: *const u64 = x;
-    RawSlice {
-        data: (x as *const u8).offset(1),
-        len: 7,
+fn inline_atom_slice_mut(x: &mut u64) -> &mut [u8] {
+    unsafe {
+        let x: *mut u64 = x;
+        let mut data = x as *mut u8;
+        // All except the lowest byte, which is first in little-endian, last in big-endian.
+        if cfg!(target_endian = "little") {
+            data = data.offset(1);
+        }
+        let len = 7;
+        slice::from_raw_parts_mut(data, len)
     }
 }
 
@@ -557,10 +538,8 @@ impl UnpackedAtom {
                 debug_assert!((len as usize) <= MAX_INLINE_LEN);
                 let mut data: u64 = (INLINE_TAG as u64) | ((len as u64) << 4);
                 {
-                    let raw_slice = inline_atom_slice(&mut data);
-                    let dest: &mut [u8] = slice::from_raw_parts_mut(
-                        raw_slice.data as *mut u8, raw_slice.len);
-                    copy_memory(&buf[..], dest);
+                    let dest = inline_atom_slice_mut(&mut data);
+                    dest.copy_from_slice(&buf)
                 }
                 data
             }
@@ -578,9 +557,8 @@ impl UnpackedAtom {
                 let len = ((data & 0xf0) >> 4) as usize;
                 debug_assert!(len <= MAX_INLINE_LEN);
                 let mut buf: [u8; 7] = [0; 7];
-                let raw_slice = inline_atom_slice(&data);
-                let src: &[u8] = slice::from_raw_parts(raw_slice.data, raw_slice.len);
-                copy_memory(src, &mut buf[..]);
+                let src = inline_atom_slice(&data);
+                buf.copy_from_slice(src);
                 Inline(len as u8, buf)
             },
             _ => debug_unreachable!(),
@@ -606,29 +584,13 @@ unsafe fn from_packed_dynamic(data: u64) -> Option<*mut ()> {
 unsafe fn inline_orig_bytes<'a>(data: &'a u64) -> &'a [u8] {
     match UnpackedAtom::from_packed(*data) {
         Inline(len, _) => {
-            let raw_slice = inline_atom_slice(&data);
-            let src: &[u8] = slice::from_raw_parts(raw_slice.data, raw_slice.len);
+            let src = inline_atom_slice(&data);
             &src[..(len as usize)]
         }
         _ => debug_unreachable!(),
     }
 }
 
-
-/// Copy of std::slice::bytes::copy_memory, which is unstable.
-#[inline]
-fn copy_memory(src: &[u8], dst: &mut [u8]) {
-    let len_src = src.len();
-    assert!(dst.len() >= len_src);
-    // `dst` is unaliasable, so we know statically it doesn't overlap
-    // with `src`.
-    unsafe {
-        ptr::copy_nonoverlapping(src.as_ptr(),
-                                 dst.as_mut_ptr(),
-                                 len_src);
-    }
-}
-
 #[cfg(test)]
 #[macro_use]
 mod tests {
